@@ -25,6 +25,9 @@
 
 #include <gweather-version.h>
 #include "gweather-location.h"
+#include "gweather-weather.h"
+
+extern void _gweather_location_reset_world (void);
 
 /* For test_metar_weather_stations */
 #define METAR_SOURCES "https://www.aviationweather.gov/docs/metar/stations.txt"
@@ -56,6 +59,8 @@ test_named_timezones (void)
         g_assert_nonnull (code);
         g_assert_true (code[0] == '@');
     }
+
+    _gweather_location_reset_world ();
 }
 
 static GList *
@@ -145,6 +150,28 @@ test_named_timezones_deserialized (void)
     g_list_free (list);
 
     test_timezones ();
+
+    _gweather_location_reset_world ();
+}
+
+static void
+test_no_code_serialize (void)
+{
+    GVariant *variant;
+    GWeatherLocation *world, *loc;
+
+    world = gweather_location_get_world ();
+    g_assert_nonnull (world);
+
+    loc = gweather_location_find_nearest_city (world, 56.833333, 53.183333);
+    g_assert (loc);
+    g_assert_cmpstr (gweather_location_get_name (loc), ==, "Izhevsk");
+
+    variant = gweather_location_serialize (loc);
+    g_assert_nonnull (variant);
+    g_variant_unref (variant);
+
+    _gweather_location_reset_world ();
 }
 
 static void
@@ -161,7 +188,7 @@ test_timezone (GWeatherLocation *location)
         g_assert (tzs);
 
         /* Only countries should have multiple timezones associated */
-        if (tzs[0] == NULL ||
+        if ((tzs[0] == NULL && gweather_location_get_level (location) < GWEATHER_LOCATION_WEATHER_STATION) &&
             gweather_location_get_level (location) > GWEATHER_LOCATION_COUNTRY) {
             g_print ("Location '%s' does not have an associated timezone\n",
                      gweather_location_get_name (location));
@@ -204,6 +231,8 @@ test_timezones (void)
     g_assert (world);
 
     test_timezones_children (world);
+
+    _gweather_location_reset_world ();
 }
 
 static void
@@ -213,6 +242,8 @@ test_distance (GWeatherLocation *location)
     double distance;
 
     parent = gweather_location_get_parent (location);
+    if (gweather_location_get_level (parent) < GWEATHER_LOCATION_CITY)
+      return;
     distance = gweather_location_get_distance (location, parent);
 
     if (distance > TOO_FAR) {
@@ -252,6 +283,8 @@ test_airport_distance_sanity (void)
 
     if (g_test_failed ())
         g_warning ("Maximum city to airport distance is %.1f km", max_distance);
+
+    _gweather_location_reset_world ();
 }
 
 static GHashTable *
@@ -376,12 +409,81 @@ test_metar_weather_stations (void)
     g_free (contents);
 
     test_metar_weather_stations_children (world, stations_ht);
+
+    _gweather_location_reset_world ();
 }
 
 static void
-check_duplicate_weather_stations (gpointer key,
-                                  gpointer value,
-                                  gpointer user_data)
+set_gsettings (void)
+{
+	char *tmpdir, *schema_text, *dest, *cmdline;
+	int result;
+
+	/* Create the installed schemas directory */
+	tmpdir = g_strdup_printf ("libgweather-test-XXXXXX");
+	tmpdir = g_dir_make_tmp (tmpdir, NULL);
+	g_assert_nonnull (tmpdir);
+
+	/* Copy the schemas files */
+	g_assert (g_file_get_contents (SCHEMAS_BUILDDIR "/org.gnome.GWeather.enums.xml", &schema_text, NULL, NULL));
+	dest = g_strdup_printf ("%s/org.gnome.GWeather.enums.xml", tmpdir);
+	g_assert (g_file_set_contents (dest, schema_text, -1, NULL));
+	g_free (dest);
+	g_free (schema_text);
+
+	g_assert (g_file_get_contents (SCHEMASDIR "/org.gnome.GWeather.gschema.xml", &schema_text, NULL, NULL));
+	dest = g_strdup_printf ("%s/org.gnome.GWeather.gschema.xml", tmpdir);
+	g_assert (g_file_set_contents (dest, schema_text, -1, NULL));
+	g_free (dest);
+	g_free (schema_text);
+
+	/* Compile the schemas */
+	cmdline = g_strdup_printf ("glib-compile-schemas --targetdir=%s "
+				   "--schema-file=%s/org.gnome.GWeather.enums.xml "
+				   "--schema-file=%s/org.gnome.GWeather.gschema.xml",
+				   tmpdir, SCHEMAS_BUILDDIR, SCHEMASDIR);
+	g_assert (g_spawn_command_line_sync (cmdline, NULL, NULL, &result, NULL));
+	g_assert (result == 0);
+	g_free (cmdline);
+
+	/* Set envvar */
+	g_setenv ("GSETTINGS_SCHEMA_DIR", tmpdir, TRUE);
+	g_setenv ("GSETTINGS_BACKEND", "memory", TRUE);
+}
+
+static void
+test_utc_sunset (void)
+{
+	GWeatherLocation *world, *utc;
+	GWeatherInfo *info;
+	char *sunset;
+	GWeatherMoonPhase phase;
+	GWeatherMoonLatitude lat;
+	gboolean ret;
+
+	world = gweather_location_get_world ();
+	g_assert_nonnull (world);
+	utc = gweather_location_find_by_station_code (world, "@UTC");
+	g_assert_nonnull (utc);
+
+	info = gweather_info_new (utc);
+	gweather_info_set_enabled_providers (info, GWEATHER_PROVIDER_NONE);
+	gweather_info_update (info);
+
+	sunset = gweather_info_get_sunset (info);
+	g_assert_nonnull (sunset);
+	g_free (sunset);
+
+	ret = gweather_info_get_value_moonphase (info, &phase, &lat);
+	g_assert_false (ret);
+
+	_gweather_location_reset_world ();
+}
+
+static void
+check_bad_duplicate_weather_stations (gpointer key,
+                                      gpointer value,
+                                      gpointer user_data)
 {
     GPtrArray *stations = value;
     GHashTable *dedup;
@@ -414,8 +516,8 @@ out:
 }
 
 static void
-test_duplicate_weather_stations_children (GWeatherLocation *location,
-                                          GHashTable       *stations_ht)
+test_bad_duplicate_weather_stations_children (GWeatherLocation *location,
+                                              GHashTable       *stations_ht)
 {
     GWeatherLocation **children;
     guint i;
@@ -434,25 +536,85 @@ test_duplicate_weather_stations_children (GWeatherLocation *location,
             g_ptr_array_add (stations, children[i]);
             g_hash_table_insert (stations_ht, g_strdup (code), stations);
         } else {
-            test_duplicate_weather_stations_children (children[i], stations_ht);
+            test_bad_duplicate_weather_stations_children (children[i], stations_ht);
         }
     }
 }
 
+static void
+test_bad_duplicate_weather_stations (void)
+{
+    GWeatherLocation *world;
+    GHashTable *stations_ht;
+
+    g_setenv ("LIBGWEATHER_LOCATIONS_NO_NEAREST", "1", TRUE);
+    world = gweather_location_get_world ();
+    g_assert (world);
+
+    stations_ht = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                         g_free, (GDestroyNotify) NULL);
+    test_bad_duplicate_weather_stations_children (world, stations_ht);
+
+    g_hash_table_foreach (stations_ht, check_bad_duplicate_weather_stations, NULL);
+
+    g_unsetenv ("LIBGWEATHER_LOCATIONS_NO_NEAREST");
+    _gweather_location_reset_world ();
+}
+
+static void
+test_duplicate_weather_stations_children (GWeatherLocation *location)
+{
+    GWeatherLocation **children;
+    GHashTable *stations_ht = NULL;
+    guint i;
+
+    children = gweather_location_get_children (location);
+    for (i = 0; children[i] != NULL; i++) {
+        if (gweather_location_get_level (children[i]) == GWEATHER_LOCATION_WEATHER_STATION) {
+            const char *code;
+
+            code = gweather_location_get_code (children[i]);
+            if (!stations_ht) {
+                stations_ht = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                     g_free, (GDestroyNotify) NULL);
+            } else {
+                gboolean exists;
+
+                exists = GPOINTER_TO_INT (g_hash_table_lookup (stations_ht, code));
+                if (exists) {
+                    GWeatherLocationLevel parent_level;
+
+                    parent_level = gweather_location_get_level (location);
+                    g_print ("Duplicate weather station '%s' in %s (level '%s')\n",
+                             code, gweather_location_get_name (location),
+                             gweather_location_level_to_string (parent_level));
+                    g_test_fail ();
+                }
+            }
+
+            g_hash_table_insert (stations_ht, g_strdup (code), GINT_TO_POINTER (1));
+        } else {
+            test_duplicate_weather_stations_children (children[i]);
+        }
+    }
+
+    if (stations_ht)
+        g_hash_table_destroy (stations_ht);
+}
+
 static void
 test_duplicate_weather_stations (void)
 {
     GWeatherLocation *world;
-    GHashTable *stations_ht;
 
+    g_setenv ("LIBGWEATHER_LOCATIONS_NO_NEAREST", "1", TRUE);
     world = gweather_location_get_world ();
     g_assert (world);
 
-    stations_ht = g_hash_table_new_full (g_str_hash, g_str_equal,
-                                         g_free, (GDestroyNotify) NULL);
-    test_duplicate_weather_stations_children (world, stations_ht);
+    test_duplicate_weather_stations_children (world);
 
-    g_hash_table_foreach (stations_ht, check_duplicate_weather_stations, NULL);
+    g_unsetenv ("LIBGWEATHER_LOCATIONS_NO_NEAREST");
+    _gweather_location_reset_world ();
 }
 
 static void
@@ -475,12 +637,17 @@ main (int argc, char *argv[])
 	g_setenv ("LIBGWEATHER_LOCATIONS_PATH",
 		  TEST_SRCDIR "../data/Locations.xml",
 		  FALSE);
+	set_gsettings ();
 
 	g_test_add_func ("/weather/named-timezones", test_named_timezones);
 	g_test_add_func ("/weather/named-timezones-deserialized", test_named_timezones_deserialized);
+	g_test_add_func ("/weather/no-code-serialize", test_no_code_serialize);
 	g_test_add_func ("/weather/timezones", test_timezones);
 	g_test_add_func ("/weather/airport_distance_sanity", test_airport_distance_sanity);
 	g_test_add_func ("/weather/metar_weather_stations", test_metar_weather_stations);
+	g_test_add_func ("/weather/utc_sunset", test_utc_sunset);
+	/* Modifies environment, so needs to run last */
+	g_test_add_func ("/weather/bad_duplicate_weather_stations", test_bad_duplicate_weather_stations);
 	g_test_add_func ("/weather/duplicate_weather_stations", test_duplicate_weather_stations);
 
 	return g_test_run ();
